/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#ifndef nspr_h___
#define nspr_h___

#include "pratom.h"
#include "prbit.h"
#include "prclist.h"
#include "prcmon.h"
#include "prcvar.h"
#include "prdtoa.h"
#include "prenv.h"
#include "prerror.h"
#include "prinet.h"
#include "prinit.h"
#include "prinrval.h"
#include "prio.h"
#include "pripcsem.h"
#include "prlink.h"
#include "prlock.h"
#include "prlog.h"
#include "prlong.h"
#include "prmem.h"
#include "prmon.h"
#include "prmwait.h"
#include "prnetdb.h"
#include "prprf.h"
#include "prproces.h"
#include "prrng.h"
#include "prrwlock.h"
#include "prshm.h"
#include "prshma.h"
#include "prsystem.h"
#include "prthread.h"
#include "prtime.h"
#include "prtpool.h"
#include "prtrace.h"
#include "prtypes.h"

#endif /* nspr_h___ */
